@@ -9,8 +9,8 @@
 #include <grp.h>
 #include <errno.h>
 
-#include "rcsid.h"
-RCSID ("$Id: addgrps.c,v 1.6 2003/04/22 10:59:21 kloczek Exp $")
+#ident "$Id: addgrps.c,v 1.9 2006/01/23 09:46:30 kloczek Exp $"
+
 #define SEP ",:"
 /*
  * Add groups with names from LIST (separated by commas or colons)
@@ -33,7 +33,7 @@ int add_groups (const char *list)
 
 	i = 16;
 	for (;;) {
-		grouplist = malloc (i * sizeof (GETGROUPS_T));
+		grouplist = (gid_t *) malloc (i * sizeof (GETGROUPS_T));
 		if (!grouplist)
 			return -1;
 		ngroups = getgroups (i, grouplist);
@@ -58,8 +58,7 @@ int add_groups (const char *list)
 			continue;
 		}
 
-		for (i = 0; i < ngroups && grouplist[i] != grp->gr_gid;
-		     i++);
+		for (i = 0; i < ngroups && grouplist[i] != grp->gr_gid; i++);
 
 		if (i < ngroups)
 			continue;
@@ -68,9 +67,7 @@ int add_groups (const char *list)
 			fprintf (stderr, _("Warning: too many groups\n"));
 			break;
 		}
-		tmp =
-		    realloc (grouplist,
-			     (ngroups + 1) * sizeof (GETGROUPS_T));
+		tmp = (gid_t *) realloc (grouplist, (ngroups + 1) * sizeof (GETGROUPS_T));
 		if (!tmp) {
 			free (grouplist);
 			return -1;
