@@ -36,8 +36,8 @@
 
 #ifndef USE_PAM
 
-#include "rcsid.h"
-RCSID ("$Id: limits.c,v 1.18 2005/06/20 10:02:02 kloczek Exp $")
+#ident "$Id: limits.c,v 1.21 2006/07/10 04:11:31 kloczek Exp $"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
@@ -138,10 +138,8 @@ static int check_logins (const char *name, const char *maxlogins)
 	setutent ();
 	while ((ut = getutent ())) {
 #endif
-#ifdef USER_PROCESS
 		if (ut->ut_type != USER_PROCESS)
 			continue;
-#endif
 		if (ut->ut_user[0] == '\0')
 			continue;
 		if (strncmp (name, ut->ut_user, sizeof (ut->ut_user)) != 0)
@@ -210,6 +208,7 @@ static int do_user_limits (const char *buf, const char *name)
 		case 'A':
 			/* RLIMIT_AS - max address space (KB) */
 			retval |= setrlimit_value (RLIMIT_AS, pp, 1024);
+			break;
 #endif
 #ifdef RLIMIT_CPU
 		case 't':
