@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: useradd.c,v 1.89 2005/12/15 15:06:28 kloczek Exp $"
+#ident "$Id: useradd.c,v 1.99 2006/06/22 11:30:32 kloczek Exp $"
 
 #include <ctype.h>
 #include <errno.h>
@@ -219,7 +219,7 @@ static long get_number (const char *cp)
 	if (*cp != '\0' && *ep == '\0')	/* valid number */
 		return val;
 
-	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog, cp);
+	fprintf (stderr, _("%s: invalid numeric argument '%s'\n"), Prog, cp);
 	exit (E_BAD_ARG);
 }
 
@@ -232,7 +232,7 @@ static uid_t get_uid (const char *cp)
 	if (*cp != '\0' && *ep == '\0')	/* valid number */
 		return val;
 
-	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog, cp);
+	fprintf (stderr, _("%s: invalid numeric argument '%s'\n"), Prog, cp);
 	exit (E_BAD_ARG);
 }
 
@@ -572,7 +572,7 @@ static int get_groups (char *list)
 		 */
 		if (__isgrNIS ()) {
 			fprintf (stderr,
-				 _("%s: group `%s' is a NIS group.\n"),
+				 _("%s: group '%s' is a NIS group.\n"),
 				 Prog, grp->gr_name);
 			continue;
 		}
@@ -633,7 +633,8 @@ static void usage (void)
 			   "  -p, --password PASSWORD	use encrypted password for the new user\n"
 			   "				account\n"
 			   "  -s, --shell SHELL		the login shell for the new user account\n"
-			   "  -u, --uid UID			force use the UID for the new user account\n"));
+			   "  -u, --uid UID			force use the UID for the new user account\n"
+			   "\n"));
 	exit (E_USAGE);
 }
 
@@ -1027,7 +1028,7 @@ static void process_flags (int argc, char **argv)
 				    || optarg[0] != '/') {
 					fprintf (stderr,
 						 _
-						 ("%s: invalid base directory `%s'\n"),
+						 ("%s: invalid base directory '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
@@ -1038,7 +1039,7 @@ static void process_flags (int argc, char **argv)
 				if (!VALID (optarg)) {
 					fprintf (stderr,
 						 _
-						 ("%s: invalid comment `%s'\n"),
+						 ("%s: invalid comment '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
@@ -1050,7 +1051,7 @@ static void process_flags (int argc, char **argv)
 				    || optarg[0] != '/') {
 					fprintf (stderr,
 						 _
-						 ("%s: invalid home directory `%s'\n"),
+						 ("%s: invalid home directory '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
@@ -1068,7 +1069,7 @@ static void process_flags (int argc, char **argv)
 					if (user_expire == -1) {
 						fprintf (stderr,
 							 _
-							 ("%s: invalid date `%s'\n"),
+							 ("%s: invalid date '%s'\n"),
 							 Prog, optarg);
 						exit (E_BAD_ARG);
 					}
@@ -1163,7 +1164,7 @@ static void process_flags (int argc, char **argv)
 				if (!VALID (optarg)) {
 					fprintf (stderr,
 						 _
-						 ("%s: invalid field `%s'\n"),
+						 ("%s: invalid field '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
@@ -1176,7 +1177,7 @@ static void process_flags (int argc, char **argv)
 					    && optarg[0] != '*'))) {
 					fprintf (stderr,
 						 _
-						 ("%s: invalid shell `%s'\n"),
+						 ("%s: invalid shell '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
@@ -1246,6 +1247,11 @@ static void process_flags (int argc, char **argv)
 
 	if (!sflg)
 		user_shell = def_shell;
+
+	/* TODO: add handle change default spool mail creation by 
+	   -K CREATE_MAIL_SPOOL={yes,no}. It need rewrite internal API for handle
+	   shadow tools configuration */
+	create_mail_spool = def_create_mail_spool;
 }
 
 /*
@@ -1271,7 +1277,6 @@ static void close_files (void)
 				 _("%s: cannot rewrite group file\n"), Prog);
 			fail_exit (E_GRP_UPDATE);
 		}
-		gr_unlock ();
 #ifdef	SHADOWGRP
 		if (is_shadow_grp && !sgr_close ()) {
 			fprintf (stderr,
@@ -1280,13 +1285,16 @@ static void close_files (void)
 				 Prog);
 			fail_exit (E_GRP_UPDATE);
 		}
-		if (is_shadow_grp)
-			sgr_unlock ();
 #endif
 	}
 	if (is_shadow_pwd)
 		spw_unlock ();
 	pw_unlock ();
+	gr_unlock ();
+#ifdef	SHADOWGRP
+	if (is_shadow_grp)
+		sgr_unlock ();
+#endif
 }
 
 /*
@@ -1564,7 +1572,8 @@ static void create_home (void)
 			fail_exit (E_HOMEDIR);
 		}
 		chown (user_home, user_id, user_gid);
-		chmod (user_home, 0777 & ~getdef_num ("UMASK", 022));
+		chmod (user_home,
+		       0777 & ~getdef_num ("UMASK", GETDEF_DEFAULT_UMASK));
 		home_added++;
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
@@ -1582,46 +1591,38 @@ static void create_home (void)
  */
 static void create_mail (void)
 {
-	char *ms;
+	char *spool, *file;
 	int fd;
-	struct group *mail;
-	gid_t mail_gid;
+	struct group *gr;
+	gid_t gid;
 	mode_t mode;
 
 	if (strcasecmp (create_mail_spool, "yes") == 0) {
-		mail = getgrnam ("mail");
-		if (mail == NULL) {
-			fprintf (stderr,
-				 _
-				 ("No group named \"mail\" exists, creating mail spool with mode 0600.\n"));
-			mode = 0600;
-			mail_gid = user_gid;
-		} else {
-			mode = 0660;
-			mail_gid = mail->gr_gid;
+		spool = getdef_str ("MAIL_DIR") ? : "/var/mail";
+		file = alloca (strlen (spool) + strlen (user_name) + 2);
+		sprintf (file, "%s/%s", spool, user_name);
+		fd = open (file, O_CREAT | O_WRONLY | O_TRUNC | O_EXCL, 0);
+		if (fd < 0) {
+			perror (_("Creating mailbox file"));
+			return;
 		}
 
-		ms = malloc (strlen (user_name) + 11);
-		if (ms != NULL) {
-			sprintf (ms, "/var/mail/%s", user_name);
-			if (access (ms, R_OK) != 0) {
-				fd = open (ms,
-					   O_CREAT | O_EXCL |
-					   O_WRONLY | O_TRUNC, 0);
-				if (fd != -1) {
-					fchown (fd, user_id, mail_gid);
-					fchmod (fd, mode);
-					close (fd);
-				}
-			} else {
-				fprintf (stderr,
-					 _
-					 ("Can't create mail spool for user %s.\n"),
-					 user_name);
-				fail_exit (E_MAIL_SPOOL);
-			}
+		gr = getgrnam ("mail");
+		if (!gr) {
+			fprintf (stderr,
+				 _
+				 ("Group 'mail' not found. Creating the user mailbox file with 0600 mode.\n"));
+			gid = user_gid;
+			mode = 0600;
+		} else {
+			gid = gr->gr_gid;
+			mode = 0660;
 		}
-		free (ms);
+
+		if (fchown (fd, user_id, gid) || fchmod (fd, mode))
+			perror (_("Setting mailbox file permissions"));
+
+		close (fd);
 	}
 }
 
@@ -1755,10 +1756,12 @@ int main (int argc, char **argv)
 	 */
 	open_files ();
 
-	/* first, seek for a valid uid to use for this user.
-	 * We do this because later we can use the uid we found as
-	 * gid too ... --gafton */
-	find_new_uid ();
+	if (!oflg) {
+		/* first, seek for a valid uid to use for this user.
+		 * We do this because later we can use the uid we found as
+		 * gid too ... --gafton */
+		find_new_uid ();
+	}
 
 	/* do we have to add a group for that user? This is why we need to
 	 * open the group files in the open_files() function  --gafton */
