@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: usermod.c,v 1.64 2005/12/05 18:19:47 kloczek Exp $"
+#ident "$Id: usermod.c,v 1.69 2006/06/25 13:10:29 kloczek Exp $"
 
 #include <ctype.h>
 #include <errno.h>
@@ -87,19 +87,21 @@ static uid_t user_newid;
 static gid_t user_gid;
 static gid_t user_newgid;
 static char *user_comment;
-static char *user_newcomment;	/* Audit */
 static char *user_home;
 static char *user_newhome;
 static char *user_shell;
-static char *user_newshell;	/* Audit */
-
 static long user_expire;
-static long user_newexpire;	/* Audit */
 static long user_inactive;
-static long user_newinactive;	/* Audit */
 static long sys_ngroups;
 static char **user_groups;	/* NULL-terminated list */
 
+#ifdef WITH_AUDIT
+static char *user_newcomment;	/* Audit */
+static char *user_newshell;	/* Audit */
+static long user_newexpire;	/* Audit */
+static long user_newinactive;	/* Audit */
+#endif
+
 static char *Prog;
 
 static int
@@ -235,7 +237,7 @@ static int get_groups (char *list)
 		 */
 		if (__isgrNIS ()) {
 			fprintf (stderr,
-				 _("%s: group `%s' is a NIS group.\n"),
+				 _("%s: group '%s' is a NIS group.\n"),
 				 Prog, grp->gr_name);
 			continue;
 		}
@@ -271,19 +273,19 @@ static int get_groups (char *list)
  */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: usermod [options] login\n"
+	fprintf (stderr, _("Usage: usermod [options] LOGIN\n"
 			   "\n"
 			   "Options:\n"
 			   "  -a, --append GROUP		append the user to the supplemental GROUP\n"
 			   "  -c, --comment COMMENT		new value of the GECOS field\n"
-			   "  -d, --home HOME_DIR		new login directory for the new user account\n"
+			   "  -d, --home HOME_DIR		new home directory for the user account\n"
 			   "  -e, --expiredate EXPIRE_DATE	set account expiration date to EXPIRE_DATE\n"
 			   "  -f, --inactive INACTIVE	set password inactive after expiration\n"
 			   "				to INACTIVE\n"
-			   "  -g, --gid GROUP		force use GROUP as new initial login group\n"
-			   "  -G, --groups GROUPS		list of supplementary GROUPS\n"
+			   "  -g, --gid GROUP		force use GROUP as new primary group\n"
+			   "  -G, --groups GROUPS		new list of supplementary GROUPS\n"
 			   "  -h, --help			display this help message and exit\n"
-			   "  -l, --login LOGIN		new value of the login name\n"
+			   "  -l, --login NEW_LOGIN		new value of the login name\n"
 			   "  -L, --lock			lock the user account\n"
 			   "  -m, --move-home		move contents of the home directory to the new\n"
 			   "				location (use only with -d)\n"
@@ -291,7 +293,8 @@ static void usage (void)
 			   "  -p, --password PASSWORD	use encrypted password for the new password\n"
 			   "  -s, --shell SHELL		new login shell for the user account\n"
 			   "  -u, --uid UID			new UID for the user account\n"
-			   "  -U, --unlock			unlock the user account\n"));
+			   "  -U, --unlock			unlock the user account\n"
+			   "\n"));
 	exit (E_USAGE);
 }
 
@@ -809,7 +812,7 @@ static long get_number (const char *cp)
 	if (*cp != '\0' && *ep == '\0')	/* valid number */
 		return val;
 
-	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog, cp);
+	fprintf (stderr, _("%s: invalid numeric argument '%s'\n"), Prog, cp);
 	exit (E_BAD_ARG);
 }
 
@@ -822,7 +825,7 @@ static uid_t get_id (const char *cp)
 	if (*cp != '\0' && *ep == '\0')	/* valid number */
 		return val;
 
-	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog, cp);
+	fprintf (stderr, _("%s: invalid numeric argument '%s'\n"), Prog, cp);
 	exit (E_BAD_ARG);
 }
 
@@ -840,7 +843,6 @@ static void process_flags (int argc, char **argv)
 
 	const struct spwd *spwd = NULL;
 	int anyflag = 0;
-	int arg;
 
 	if (argc == 1 || argv[argc - 1][0] == '-')
 		usage ();
@@ -920,7 +922,7 @@ static void process_flags (int argc, char **argv)
 			{NULL, 0, NULL, '\0'}
 		};
 		while ((c =
-			getopt_long (argc, argv, "ac:d:e:f:g:G:l:Lmop:s:u:U",
+			getopt_long (argc, argv, "a:c:d:e:f:g:G:hl:Lmop:s:u:U",
 				     long_options, NULL)) != -1) {
 			switch (c) {
 			case 'a':
@@ -929,7 +931,7 @@ static void process_flags (int argc, char **argv)
 			case 'c':
 				if (!VALID (optarg)) {
 					fprintf (stderr,
-						 _("%s: invalid field `%s'\n"),
+						 _("%s: invalid field '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
@@ -943,7 +945,7 @@ static void process_flags (int argc, char **argv)
 			case 'd':
 				if (!VALID (optarg)) {
 					fprintf (stderr,
-						 _("%s: invalid field `%s'\n"),
+						 _("%s: invalid field '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
@@ -961,7 +963,7 @@ static void process_flags (int argc, char **argv)
 #endif
 						fprintf (stderr,
 							 _
-							 ("%s: invalid date `%s'\n"),
+							 ("%s: invalid date '%s'\n"),
 							 Prog, optarg);
 						exit (E_BAD_ARG);
 					}
@@ -1005,7 +1007,7 @@ static void process_flags (int argc, char **argv)
 			case 'l':
 				if (!check_user_name (optarg)) {
 					fprintf (stderr,
-						 _("%s: invalid field `%s'\n"),
+						 _("%s: invalid field '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
@@ -1048,7 +1050,7 @@ static void process_flags (int argc, char **argv)
 			case 's':
 				if (!VALID (optarg)) {
 					fprintf (stderr,
-						 _("%s: invalid field `%s'\n"),
+						 _("%s: invalid field '%s'\n"),
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
